/* Foxhash.fll   vrs.fll (. )								*/
/* Updated 29.08.2005														*/
/*    MD5, SHA1, SHA256     .*/
/* :																	*/
/*	MD5Hash -  MD5  										*/
/*	. - .													*/
/*																			*/
/*	MD5HashFile	-  MD5  									*/
/*	. -  .												*/
/*																			*/
/*	MD5VerifyFile -  .											*/
/*	. -  , MD5  (   !).			*/
/*																			*/
/*==========================================================================*/
/* vrs.fll																	*/
/* MD5 Encryption Algorithm FLL												*/
/* Last Updated 8/25/98 													*/
/* Written by Peter Stephens for											*/
/*  Lotherington & Parker, Inc												*/
/* Copyright 1998 Lotherington & Parker, Inc.								*/
//==============================================================================
// Standard FoxPro API include file
#include <pro_ext.h>
//#include <memory.h>
#include <string.h>
#include "md5.h"
#include "sha1.h"
#include "sha256.h"
#include "hashlib.h"

#define p0 parm->p[0]
#define p1 parm->p[1]

//==============================================================================
// void MD5Hash(ParamBlk *parm)
// PARAMETERS:
//  1: C Input String for hashing
// RETURNS:
//  a text 32 byte hash value
// EXAMPLE:
//  ? MD5Hash("Hello World")
//==============================================================================
/*
void MD5Hash(ParamBlk *parm)
{
	Value val;
	MHANDLE handle;
	unsigned char strHashOut[32];	// Text of MD5 sum

	val.ev_type = 'C'; 
	val.ev_length = '0';			// Return .null. by default

	if(p0.val.ev_type == 'C') {
		handle = _AllocHand(16);
		if(handle==0) {
			_RetVal(&val);
			return;
		}

		md5_context context;

		// Start MD5 Hashing
		md5_starts(&context);
		_HLock(p0.val.ev_handle);
		md5_update(&context, (unsigned char *)_HandToPtr(p0.val.ev_handle), p0.val.ev_length);
		_HUnLock(p0.val.ev_handle);
		_HLock(handle);
		md5_finish(&context, (unsigned char *)_HandToPtr(handle));
		md5_print((unsigned char *)_HandToPtr(handle),strHashOut); // Convert to Text (ASK)
		_HUnLock(handle);
		handle = _AllocHand(32);
		if(handle==0) {
			_RetVal(&val);
			return;
		}
		_HLock(handle);
		memcpy(_HandToPtr(handle), strHashOut, 32);
		_HUnLock(handle);
		val.ev_type = 'C';
		//val.ev_length = 16;
		val.ev_length = 32;
		val.ev_handle = handle;
		// Success

		_RetVal(&val);
		return;
	} else {
		_RetVal(&val);
		return;
	}
}
*/

//==============================================================================
// void MD5_Hash(ParamBlk *parm)
// PARAMETERS:
//  1: C Input String for hashing
// RETURNS:
//  a text 32 byte hash value
// EXAMPLE:
//  ? MD5_Hash("Hello World")
//==============================================================================

void MD5_Hash(ParamBlk *parm)
{
	Value val;
	MHANDLE handle;
	unsigned char strHashOut[32];	// Text of MD5 sum

	val.ev_type = 'C'; 
	val.ev_length = '0';			// Return .null. by default

	if(p0.val.ev_type == 'C') {
		handle = _AllocHand(32);
		if(handle==0) {
			_RetVal(&val);
			return;
		}

		// Start MD5 Hashing
		_HLock(p0.val.ev_handle);
		md5_string((unsigned char *)_HandToPtr(p0.val.ev_handle), p0.val.ev_length, strHashOut);
		_HUnLock(p0.val.ev_handle);

		_HLock(handle);
		memcpy(_HandToPtr(handle), strHashOut, 32);
		_HUnLock(handle);

		val.ev_type = 'C';
		val.ev_length = 32;
		val.ev_handle = handle;
		// Success
	}

	_RetVal(&val);
	return;
}
/*
//==============================================================================
// void MD5_VerifyFile(ParamBlk *parm)
// PARAMETERS:
//  1: C Input FileName for verify
//  2: C Input MD5 Sum
// RETURNS:
//  a logical value
//==============================================================================
void MD5_VerifyFile(ParamBlk* parm)
{
	char *strzFileName;
	unsigned char strHashSrc[16];
	unsigned char strHashOut[32]; // Text of MD5 sum
	int RetValue;
	
	// Determine File Name
	if(p0.val.ev_type == 'C') {
		_HLock(p0.val.ev_handle);
		strzFileName = (char *)malloc( (size_t)p0.val.ev_length + 1 );
		if(strzFileName == NULL) {
			_RetLogical(0);
			return;
		}
		memcpy(strzFileName, _HandToPtr(p0.val.ev_handle), (size_t)p0.val.ev_length);
		_HUnLock(p0.val.ev_handle);
		strzFileName[p0.val.ev_length] = 0;
	} else {
		_RetLogical(0);
		return;
	}

	// Determine mode
	char *strBuff;
	DWORD nBuffSize, nStart;
	if(parm->pCount >= 2) {
		// The comparative hash key is being passed to this function
		if(p1.val.ev_type != 'C' || p1.val.ev_length != 32) {
			free(strzFileName);
			_RetLogical(0);
			return;
		}
		//_HLock(p1.val.ev_handle);
		//memcpy(strHashCmp, _HandToPtr(p1.val.ev_handle), 32);
		//_HUnLock(p1.val.ev_handle);
		GetFile(strzFileName, &strBuff, &nBuffSize);
		free(strzFileName);
		nStart = 0;
	} else {
		// Determine comparative hash key from file
		GetFile(strzFileName, &strBuff, &nBuffSize);
		free(strzFileName);
		if(strBuff == NULL || nBuffSize < 34) // {
		{ 
			_RetLogical(0);
			return;
		}
		//ReadHashKey(strHashCmp, strBuff);
		nStart = 34;
	}

	// Hash the buffer
	md5_context context;

	// Start MD5 Hashing
	md5_starts(&context);
	md5_update(&context, (unsigned char *)strBuff + nStart, nBuffSize - nStart);
	md5_finish(&context, (unsigned char *)strHashSrc);
	free(strBuff);

	md5_print((unsigned char *)strHashSrc,strHashOut);

	// Compare
	//if(memcmp(char *)strHashOut, strHashCmp, 32) == 0)
	//	_RetLogical(1);
	//else
	//	_RetLogical(0);

	_HLock(p1.val.ev_handle);
	RetValue = _MemCmp(strHashOut, _HandToPtr(p1.val.ev_handle), 32);
	_HUnLock(p1.val.ev_handle);

	if(RetValue == 0)
		_RetLogical(1);
	else
		_RetLogical(0);

	return;
}
*/
//==============================================================================
// void MD5_HashFile(ParamBlk *parm)
// PARAMETERS:
//  1: C Input FileName for hashing
// RETURNS:
//  a text 32 byte hash value
// EXAMPLE:
//  ? MD5HashFile("C:\config.sys")
//==============================================================================
void MD5_HashFile(ParamBlk* parm)
{
	Value val;
	MHANDLE handle;
	char *strzFileName, *strBuff;
	unsigned char strHashSrc[16];
	unsigned char strHashOut[32]; // Text of MD5 sum
	DWORD nBuffSize;

	val.ev_type = 'C';  // Return .null. by default
	val.ev_length = '0';
	
	// Determine File Name
	if(p0.val.ev_type == 'C') {
		_HLock(p0.val.ev_handle);
		strzFileName = (char *)malloc( (size_t)p0.val.ev_length + 1 );
		if(strzFileName == NULL) {
			_RetVal(&val);
			return;
		}
		memcpy(strzFileName, _HandToPtr(p0.val.ev_handle), (size_t)p0.val.ev_length);
		_HUnLock(p0.val.ev_handle);
		strzFileName[p0.val.ev_length] = 0;
	} else {
		_RetVal(&val);
		return;
	}

	// Read file
	strBuff = GetFile(strzFileName, &strBuff, &nBuffSize);
	free(strzFileName);

	if(strBuff == NULL) {
		// Error reading file
		_RetVal(&val);
		return;
	}

	// Hash the buffer
	md5_context context;

	md5_starts(&context);
	md5_update(&context, (unsigned char *)strBuff, nBuffSize);
	md5_finish(&context, (unsigned char *)strHashSrc);
	free(strBuff);
	md5_print((unsigned char *)strHashSrc, strHashOut);

	//handle = _AllocHand(16);
	handle = _AllocHand(32);
	if(handle==0) {
		_RetVal(&val);
		return;
	}

	_HLock(handle);
	memcpy(_HandToPtr(handle), strHashOut, 32);
	_HUnLock(handle);

	val.ev_type = 'C';
	//val.ev_length = 16;
	val.ev_length = 32;
	val.ev_handle = handle;
	// Success

	_RetVal(&val);
	return;
}

//==============================================================================
// void SHA1_Hash(ParamBlk *parm)
// PARAMETERS:
//  1: C Input String for hashing
// RETURNS:
//  a text 40 byte hash value
// EXAMPLE:
//  ? SHA1Hash("Hello World")
//==============================================================================

void SHA1_Hash(ParamBlk *parm)

{
	Value val;
	MHANDLE handle;
	unsigned char strHashOut[41];	// Text of SHA1 sum
/*
	strHashOut    41.   40 -   .
	   :((( 
*/
	val.ev_type = 'C'; 
	val.ev_length = '0';			// Return .null. by default

	if(p0.val.ev_type == 'C') {
		handle = _AllocHand(40);
		if(handle==0) {
			_RetVal(&val);
			return;
		}

		// Start SHA1 Hashing
		_HLock(p0.val.ev_handle);
		sha1_string((unsigned char *)_HandToPtr(p0.val.ev_handle), p0.val.ev_length, strHashOut);
		_HUnLock(p0.val.ev_handle);

		_HLock(handle);
		memcpy(_HandToPtr(handle), strHashOut, 40);
		_HUnLock(handle);

		val.ev_type = 'C';
		val.ev_length = 40;
		val.ev_handle = handle;
		// Success
	}

	_RetVal(&val);
	return;
}

//==============================================================================
// void SHA1_HashFile(ParamBlk *parm)
// PARAMETERS:
//  1: C Input FileName for hashing
// RETURNS:
//  a text 40 byte hash value
// EXAMPLE:
//  ? SHA1HashFile("C:\config.sys")
//==============================================================================
void SHA1_HashFile(ParamBlk* parm)
{
	Value val;
	MHANDLE handle;
	char *strzFileName, *strBuff;
	unsigned char strHashSrc[20];
	unsigned char strHashOut[40]; // Text of SHA1 sum
	DWORD nBuffSize;

	val.ev_type = 'C';  // Return .null. by default
	val.ev_length = '0';
	
	// Determine File Name
	if(p0.val.ev_type == 'C') {
		_HLock(p0.val.ev_handle);
		strzFileName = (char *)malloc( (size_t)p0.val.ev_length + 1 );
		if(strzFileName == NULL) {
			_RetVal(&val);
			return;
		}
		memcpy(strzFileName, _HandToPtr(p0.val.ev_handle), (size_t)p0.val.ev_length);
		_HUnLock(p0.val.ev_handle);
		strzFileName[p0.val.ev_length] = 0;
	} else {
		_RetVal(&val);
		return;
	}

	// Read file
	strBuff = GetFile(strzFileName, &strBuff, &nBuffSize);
	free(strzFileName);

	if(strBuff == NULL) {
		// Error reading file
		_RetVal(&val);
		return;
	}

	// Hash the buffer
	sha1_context context;

	sha1_starts(&context);
	sha1_update(&context, (unsigned char *)strBuff, nBuffSize);
	sha1_finish(&context, (unsigned char *)strHashSrc);
	free(strBuff);
	sha1_print((unsigned char *)strHashSrc,strHashOut);

	handle = _AllocHand(40);
	if(handle==0) {
		_RetVal(&val);
		return;
	}

	_HLock(handle);
	memcpy(_HandToPtr(handle), strHashOut, 40);
	_HUnLock(handle);

	val.ev_type = 'C';
	val.ev_length = 40;
	val.ev_handle = handle;
	// Success

	_RetVal(&val);
	return;
}

//==============================================================================
// void SHA2_Hash(ParamBlk *parm)
// PARAMETERS:
//  1: C Input String for hashing
// RETURNS:
//  a text 64 byte hash value
// EXAMPLE:
//  ? SHA2Hash("Hello World")
//==============================================================================

void SHA2_Hash(ParamBlk *parm)
{
	Value val;
	MHANDLE handle;
	unsigned char strHashOut[65];	// Text of SHA256 sum
/*
	strHashOut    65.   64 -   .
	   :((( 
*/
	val.ev_type = 'C'; 
	val.ev_length = '0';			// Return .null. by default

	if(p0.val.ev_type == 'C') {
		handle = _AllocHand(64);
		if(handle==0) {
			_RetVal(&val);
			return;
		}

		// Start SHA256 Hashing
		_HLock(p0.val.ev_handle);
		sha256_string((unsigned char *)_HandToPtr(p0.val.ev_handle), p0.val.ev_length, strHashOut);
		_HUnLock(p0.val.ev_handle);

		_HLock(handle);
		memcpy(_HandToPtr(handle), strHashOut, 64);
		_HUnLock(handle);

		val.ev_type = 'C';
		val.ev_length = 64;
		val.ev_handle = handle;
		// Success
	}

	_RetVal(&val);
	return;
}

//==============================================================================
// void SHA2_HashFile(ParamBlk *parm)
// PARAMETERS:
//  1: C Input FileName for hashing
// RETURNS:
//  a text 64 byte hash value
// EXAMPLE:
//  ? SHA2HashFile("C:\config.sys")
//==============================================================================
void SHA2_HashFile(ParamBlk* parm)
{
	Value val;
	MHANDLE handle;
	char *strzFileName, *strBuff;
	unsigned char strHashSrc[32];
	unsigned char strHashOut[64]; // Text of SHA256 sum
	DWORD nBuffSize;

	val.ev_type = 'C';  // Return .null. by default
	val.ev_length = '0';
	
	// Determine File Name
	if(p0.val.ev_type == 'C') {
		_HLock(p0.val.ev_handle);
		strzFileName = (char *)malloc( (size_t)p0.val.ev_length + 1 );
		if(strzFileName == NULL) {
			_RetVal(&val);
			return;
		}
		memcpy(strzFileName, _HandToPtr(p0.val.ev_handle), (size_t)p0.val.ev_length);
		_HUnLock(p0.val.ev_handle);
		strzFileName[p0.val.ev_length] = 0;
	} else {
		_RetVal(&val);
		return;
	}

	// Read file
	strBuff = GetFile(strzFileName, &strBuff, &nBuffSize);
	free(strzFileName);

	if(strBuff == NULL) {
		// Error reading file
		_RetVal(&val);
		return;
	}

	// Hash the buffer
	sha256_context context;

	sha256_starts(&context);
	sha256_update(&context, (unsigned char *)strBuff, nBuffSize);
	sha256_finish(&context, (unsigned char *)strHashSrc);
	free(strBuff);
	sha256_print((unsigned char *)strHashSrc,strHashOut);

	handle = _AllocHand(64);
	if(handle==0) {
		_RetVal(&val);
		return;
	}

	_HLock(handle);
	memcpy(_HandToPtr(handle), strHashOut, 64);
	_HUnLock(handle);

	val.ev_type = 'C';
	val.ev_length = 64;
	val.ev_handle = handle;
	// Success

	_RetVal(&val);
	return;
}

/*
void MD5Version(ParamBlk* parm)
{
	_RetChar("Visual Records Security Version 1.1\n"
		"Copyright (C) 1998, 1999 Visual Records, Inc.");
}
*/

//==============================================================================
// void Verify_HashFile(ParamBlk *parm)
// PARAMETERS:
//  1: C Input FileName for verify
//  2: C Input Hash Sum
// RETURNS:
//  a logical value
//==============================================================================
void Verify_HashFile(ParamBlk* parm)
{
	char *strzFileName;
	unsigned char strHashSrc[32];
	unsigned char strHashOut[64]; // Text of hash sum
	int LenHash, RetValue;
	
	// Determine File Name
	if(p0.val.ev_type == 'C') {
		_HLock(p0.val.ev_handle);
		strzFileName = (char *)malloc( (size_t)p0.val.ev_length + 1 );
		if(strzFileName == NULL) {
			_RetLogical(0);
			return;
		}
		memcpy(strzFileName, _HandToPtr(p0.val.ev_handle), (size_t)p0.val.ev_length);
		_HUnLock(p0.val.ev_handle);
		strzFileName[p0.val.ev_length] = 0;
	} else {
		_RetLogical(0);
		return;
	}

	LenHash = p1.val.ev_length;
	
	// Determine mode
	char *strBuff;
	DWORD nBuffSize, nStart;
	if(parm->pCount >= 2) {
		// The comparative hash key is being passed to this function
		if(p1.val.ev_type != 'C' || \
			LenHash < 32 || \
			LenHash > 64) {
			free(strzFileName);
			_RetLogical(0);
			return;
		}
		//_HLock(p1.val.ev_handle);
		//memcpy(strHashCmp, _HandToPtr(p1.val.ev_handle), 32);
		//_HUnLock(p1.val.ev_handle);
		GetFile(strzFileName, &strBuff, &nBuffSize);
		free(strzFileName);
		nStart = 0;
	} else {
		// Determine comparative hash key from file
		GetFile(strzFileName, &strBuff, &nBuffSize);
		free(strzFileName);
		if(strBuff == NULL || nBuffSize < 34) // {
		{ 
			_RetLogical(0);
			return;
		}
		//ReadHashKey(strHashCmp, strBuff);
		nStart = 34;
	}

	 switch (LenHash)
        {
            case 32:
				// Hash the buffer
				md5_context ctx5;
				// Start MD5 Hashing
				md5_starts(&ctx5);
				md5_update(&ctx5, (unsigned char *)strBuff + nStart, nBuffSize - nStart);
				md5_finish(&ctx5, (unsigned char *)strHashSrc);
				free(strBuff);
				md5_print((unsigned char *)strHashSrc,strHashOut);
				// Compare
				_HLock(p1.val.ev_handle);
				RetValue = _MemCmp(strHashOut, _HandToPtr(p1.val.ev_handle), 32);
				_HUnLock(p1.val.ev_handle);
				break;
			case 40:
				// Hash the buffer
				sha1_context ctx1;
				// Start SHA1 Hashing
				sha1_starts(&ctx1);
				sha1_update(&ctx1, (unsigned char *)strBuff + nStart, nBuffSize - nStart);
				sha1_finish(&ctx1, (unsigned char *)strHashSrc);
				free(strBuff);
				sha1_print((unsigned char *)strHashSrc,strHashOut);
				// Compare
				_HLock(p1.val.ev_handle);
				RetValue = _MemCmp(strHashOut, _HandToPtr(p1.val.ev_handle), 40);
				_HUnLock(p1.val.ev_handle);
				break;
			case 64:
				// Hash the buffer
				sha256_context ctx2;
				// Start SHA256 Hashing
				sha256_starts(&ctx2);
				sha256_update(&ctx2, (unsigned char *)strBuff + nStart, nBuffSize - nStart);
				sha256_finish(&ctx2, (unsigned char *)strHashSrc);
				free(strBuff);
				sha256_print((unsigned char *)strHashSrc,strHashOut);
				// Compare
				_HLock(p1.val.ev_handle);
				RetValue = _MemCmp(strHashOut, _HandToPtr(p1.val.ev_handle), 64);
				_HUnLock(p1.val.ev_handle);
				break;
			default:
				RetValue = 0;
		}

	if(RetValue == 0)
		_RetLogical(1);
	else
		_RetLogical(0);

	return;
}

// the FoxInfo structure registers the function
FoxInfo MD5FoxInfo[] = {
    {"MD5HASH",			(FPFI) MD5_Hash,		1, "C"}, 
	{"MD5HASHFILE",		(FPFI) MD5_HashFile,	1, "C"}, // Returns the MD5 hash value for a file
	{"SHA1HASH",		(FPFI) SHA1_Hash,		1, "C"},
	{"SHA1HASHFILE",	(FPFI) SHA1_HashFile,	1, "C"}, // Returns the SHA1 hash value for a file
	{"SHA2HASH",		(FPFI) SHA2_Hash,		1, "C"},
	{"SHA2HASHFILE",	(FPFI) SHA2_HashFile,	1, "C"}, // Returns the SHA256 hash value for a file
	{"VERIFYHASHFILE",	(FPFI) Verify_HashFile,	2, "C.C"}, // Returns boolean whether the file is valid
	//{"MD5VERIFYFILE",	(FPFI) MD5_VerifyFile,	2, "C.C"}, // Returns boolean whether the file is valid
	//{"MD5VERSION",       (FPFI) MD5Version,      0, ""},
};

extern "C" {
    // the FoxTable structure
    FoxTable _FoxTable = {
        (FoxTable  *) 0, sizeof(MD5FoxInfo)/sizeof(FoxInfo), MD5FoxInfo
    };
}
